package org.dromara.system.controller.register;

import java.util.List;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.annotation.SaIgnore;
import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import lombok.RequiredArgsConstructor;
import jakarta.servlet.http.HttpServletResponse;
import jakarta.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.dromara.common.core.constant.TenantConstants;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import org.dromara.common.idempotent.annotation.RepeatSubmit;
import org.dromara.common.log.annotation.Log;
import org.dromara.common.web.core.BaseController;
import org.dromara.common.mybatis.core.page.PageQuery;
import org.dromara.common.core.domain.R;
import org.dromara.common.core.validate.AddGroup;
import org.dromara.common.core.validate.EditGroup;
import org.dromara.common.log.enums.BusinessType;
import org.dromara.common.excel.utils.ExcelUtil;
import org.dromara.system.domain.vo.RegisterTeacherVo;
import org.dromara.system.domain.bo.RegisterTeacherBo;
import org.dromara.system.service.IRegisterTeacherService;
import org.dromara.common.mybatis.core.page.TableDataInfo;

/**
 * 注册-老师
 *
 * @author boman
 * @date 2023-08-16
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/register/teacher")
public class RegisterTeacherController extends BaseController {

    private final IRegisterTeacherService registerTeacherService;

    /**
     * 查询注册-老师列表
     */
    @SaCheckRole(TenantConstants.SUPER_ADMIN_ROLE_KEY)
    @GetMapping("/list")
    public TableDataInfo<RegisterTeacherVo> list(RegisterTeacherBo bo, PageQuery pageQuery) {
        return registerTeacherService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出注册-老师列表
     */
    @SaCheckPermission("register:teacher:export")
    @Log(title = "注册-老师", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(RegisterTeacherBo bo, HttpServletResponse response) {
        List<RegisterTeacherVo> list = registerTeacherService.queryList(bo);
        ExcelUtil.exportExcel(list, "注册-老师", RegisterTeacherVo.class, response);
    }

    /**
     * 获取注册-老师详细信息
     *
     * @param id 主键
     */
    @SaCheckPermission("register:teacher:query")
    @GetMapping("/{id}")
    public R<RegisterTeacherVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable Long id) {
        return R.ok(registerTeacherService.queryById(id));
    }

    /**
     * 新增注册-老师
     */
    @Log(title = "注册-老师", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<RegisterTeacherVo> add(@Validated(AddGroup.class) @RequestBody RegisterTeacherBo bo) {
        return registerTeacherService.insertByBo(bo);
    }

    /**
     * 修改注册-老师
     */
    @Log(title = "注册-老师", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PostMapping("/put")
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody RegisterTeacherBo bo) {
        return toAjax(registerTeacherService.updateByBo(bo));
    }

    /**
     * 删除注册-老师
     *
     * @param ids 主键串
     */
    @SaCheckPermission("register:teacher:remove")
    @Log(title = "注册-老师", businessType = BusinessType.DELETE)
    @GetMapping("/delete/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable Long[] ids) {
        return toAjax(registerTeacherService.deleteWithValidByIds(ids, true));
    }
}
