package org.dromara.system.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import org.dromara.common.mybatis.annotation.DataColumn;
import org.dromara.common.mybatis.annotation.DataPermission;
import org.dromara.common.mybatis.core.mapper.BaseMapperPlus;
import org.dromara.system.domain.SysDept;
import org.dromara.system.domain.bo.SysDeptBo;
import org.dromara.system.domain.vo.SysDeptVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 部门管理 数据层
 *
 * @author Lion Li
 */
public interface SysDeptMapper extends BaseMapperPlus<SysDept, SysDeptVo> {

    /**
     * 查询部门管理数据
     *
     * @param queryWrapper 查询条件
     * @return 部门信息集合
     */
    @DataPermission({
        @DataColumn(key = "deptName", value = "dept_id")
    })
    List<SysDeptVo> selectDeptList(@Param(Constants.WRAPPER) Wrapper<SysDept> queryWrapper);

    @DataPermission({
        @DataColumn(key = "deptName", value = "dept_id")
    })
    SysDeptVo selectDeptById(Long deptId);

    /**
     * 根据角色ID查询部门树信息
     *
     * @param roleId            角色ID
     * @param deptCheckStrictly 部门树选择项是否关联显示
     * @return 选中部门列表
     */
    List<Long> selectDeptListByRoleId(@Param("roleId") Long roleId, @Param("deptCheckStrictly") boolean deptCheckStrictly);

    @InterceptorIgnore(tenantLine = "true")
    SysDeptVo selectDeptByParentId(long parentId);

    @InterceptorIgnore(tenantLine = "true")
    void insertDeptList(List<SysDept> sysDeptList);

    @InterceptorIgnore(tenantLine = "true")
    void updateDeptIsStatus(Long parentId);

    @InterceptorIgnore(tenantLine = "true")
    List<SysDeptVo> selectDeptListChildren(Long parentId);

    @InterceptorIgnore(tenantLine = "true")
    List<SysDeptVo> selectDeptOldList(SysDeptBo dept);

    @InterceptorIgnore(tenantLine = "true")
    List<SysDeptVo> selectDeptListMapper(SysDeptBo dept);

    @InterceptorIgnore(tenantLine = "true")
    SysDeptVo selectDeptByIdNoTenantId(Long deptId);

    @InterceptorIgnore(tenantLine = "true")
    List<SysDeptVo> selectDeptListMapperByIdList(List<Long> deptIdList);

    List<SysDeptVo> selectDeptSelfListMapper(SysDeptBo dept);
}
